	/**
	 *	Draws box plot in a canvas context of percentile data
	 *	supplied as a JSON object.
	 */
	function createBoxPlot(boxPlotJsonObject, canvas, draw_area_wdt, draw_area_hgt)
	{
		var ctx = canvas.getContext('2d');
		ctx.clearRect(0, 0, draw_area_wdt, draw_area_hgt);
		// Reserve area for scale and titles
		var headerTitleHgt = 25;
		var drect_xoff = 80;
		var drect_yoff = 90 + headerTitleHgt;
		var header_text_top_off = 5;
		var text_axis_off = 55;
		// Calculate remaining plot area
		var drect_w = draw_area_wdt - 2*drect_xoff;
		var drect_h = draw_area_hgt - 2*drect_yoff;
		// Draw rectangle enclosing plot area
		ctx.strokeStyle = "#000000";
		ctx.strokeRect(drect_xoff, drect_yoff, drect_w, drect_h);
		// Add top header title 
		var headerTitleTop = getJSONData(boxPlotJsonObject, 'headerTitleTop');
		drawHeaderTitleTopCenter(headerTitleTop, header_text_top_off, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		// Add top plot title 
		var titleTop = getJSONData(boxPlotJsonObject, 'titleTop');
		drawTitleTopCenter(titleTop, text_axis_off, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		// Add bottom plot title 
		var titleBottom = getJSONData(boxPlotJsonObject, 'titleBottom');
		drawTitleBottomCenter(titleBottom, text_axis_off, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		// Add plot top right sub-title
		var subTitleRight = getJSONData(boxPlotJsonObject, 'subTitleRight');
		drawSubTitleTopRight(subTitleRight, text_axis_off, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		// Add plot top left sub-titles
		var subTitleLeft01 = getJSONData(boxPlotJsonObject, 'subTitleLeft01');
		var subTitleLeft02 = getJSONData(boxPlotJsonObject, 'subTitleLeft02');
		var subTitleLeft03 = getJSONData(boxPlotJsonObject, 'subTitleLeft03');
		drawSubTitleTopLeft(subTitleLeft01, 0, text_axis_off, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		drawSubTitleTopLeft(subTitleLeft02, 1, text_axis_off, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		drawSubTitleTopLeft(subTitleLeft03, 2, text_axis_off, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		// Add y-axis left title
		var yAxisTitleLeft = getJSONData(boxPlotJsonObject, 'yAxisTitleLeft');
		drawTitleYLeft(yAxisTitleLeft, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		// Add y-axis right title
		var yAxisTitleRight = getJSONData(boxPlotJsonObject, 'yAxisTitleRight');
		drawTitleYRight(yAxisTitleRight, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		// Calculate y-axis data value limits
		var minDataValue = calculateMinDataValue(boxPlotJsonObject);
		var maxDataValue = calculateMaxDataValue(boxPlotJsonObject);
		// Calculate scale marker array
		var numTicks = 7;
		var scaleMarkerArr = calculateScaleMarkerArray(minDataValue, maxDataValue, numTicks);
		// Add 2% buffer below min value and 5% buffer above max value
		var dataValueRange = maxDataValue - minDataValue;
		var min_y = minDataValue - 0.02*dataValueRange;
		var max_y = maxDataValue + 0.05*dataValueRange;
		// Trim scale marker array to only include values inside range
		var trimmedScaleMarkerArr = trimScaleMarkerArray(scaleMarkerArr, min_y, max_y);
		// Add small spacing buffer in pixels below and above value area
		var min_yoff = 5;
		var max_yoff = 5;
		// Select number of decimals for scale marker text
		// Select number of decimals to get one decimal digit different from 0, up to 3 decimals
		var numberOfDecimals = 0;
		for (i=0; i < trimmedScaleMarkerArr.length; i++)
		{
			var scaleValueTmp = trimmedScaleMarkerArr[i];
			// Round scale value to 3 decimals (to make e.g. 2.99999993 -> 3.000)
			scaleValueTmp = scaleValueTmp.toFixed(3);
			var decimalPart = scaleValueTmp - Math.floor(scaleValueTmp);
			if (decimalPart >= 0.05)
			{
				if (numberOfDecimals == 0 || numberOfDecimals > 1)
				{
					numberOfDecimals = 1;
				}
			}
			else if (decimalPart >= 0.005)
			{
				if (numberOfDecimals == 0 || numberOfDecimals > 2)
				{
					numberOfDecimals = 2;
				}
			}
			else if (decimalPart >= 0.0005)
			{
				if (numberOfDecimals == 0 || numberOfDecimals > 3)
				{
					numberOfDecimals = 3;
				}
			}
		}
		// Draw y-axis left scale markers with values
		var scaleValue;
		var markerText;
		for (i=0; i < trimmedScaleMarkerArr.length; i++)
		{
			scaleValue = trimmedScaleMarkerArr[i];
			markerText = scaleValue.toFixed(numberOfDecimals);
			drawScaleMarkerYLeft(scaleValue, markerText, min_y, max_y, min_yoff, max_yoff, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		}
		// Find number of boxes in box plot
		var boxNumTot = 0;
		for (var i in boxPlotJsonObject.percentileData)
		{
			boxNumTot++;
		}
		// Draw box plot boxes
		for (var i in boxPlotJsonObject.percentileData)
		{
			var boxNum = i;
			var name = boxPlotJsonObject.percentileData[i].name;
			var numItems = boxPlotJsonObject.percentileData[i].numItems;
			var v1 = boxPlotJsonObject.percentileData[i].v1;
			var v2 = boxPlotJsonObject.percentileData[i].v2;
			var v3 = boxPlotJsonObject.percentileData[i].v3;
			var v4 = boxPlotJsonObject.percentileData[i].v4;
			var v5 = boxPlotJsonObject.percentileData[i].v5;
			// Draw box plot box
			var x_pad = 10;
			drawBoxPlotBoxY(boxNum, boxNumTot, v1, v2, v3, v4, v5, min_y, max_y, min_yoff, max_yoff, x_pad, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
			drawBoxPlotBoxScaleMarkerXBottom(boxNum, boxNumTot, name, x_pad, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
			drawBoxPlotBoxScaleMarkerXTop(boxNum, boxNumTot, numItems, x_pad, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		}
		// Draw y-axis right scale markers with values and horizontal dotted guide lines
		for (var i in boxPlotJsonObject.valueGuideLinesY)
		{
			var value = boxPlotJsonObject.valueGuideLinesY[i].value;
			var valueText = boxPlotJsonObject.valueGuideLinesY[i].text;
			// Draw y-axis right scale markers with values
			drawScaleMarkerYRight(value, valueText, min_y, max_y, min_yoff, max_yoff, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
			// Draw dotted horizontal lines at y axis right scale marker values
			drawDottedLineY(value, min_y, max_y, min_yoff, max_yoff, ctx, drect_xoff, drect_yoff, drect_w, drect_h);
		}
	}
	 
	/**
	 *	Calculate min data value for percentiles in box plot JSON object.
	 */
	function calculateMinDataValue(boxPlotJsonObject)
	{
		// Find number of boxes in box plot
		var boxNumTot = 0;
		for (var i in boxPlotJsonObject.percentileData)
		{
			boxNumTot++;
		}
		// Calculate min percentile value
		var minValue = 1000000000;
		for (var i in boxPlotJsonObject.percentileData)
		{
			var boxNum = i;
			var v1 = boxPlotJsonObject.percentileData[i].v1;
			var v5 = boxPlotJsonObject.percentileData[i].v5;
			if (v1 < minValue)
			{
				minValue = v1;
			}
			if (v5 < minValue)
			{
				minValue = v5;
			}
		}
		return minValue;
	}
	 
	/**
	 *	Calculate max data value for percentiles in box plot JSON object.
	 */
	function calculateMaxDataValue(boxPlotJsonObject)
	{
		// Find number of boxes in box plot
		var boxNumTot = 0;
		for (var i in boxPlotJsonObject.percentileData)
		{
			boxNumTot++;
		}
		// Calculate max percentile value
		var maxValue = -1;
		for (var i in boxPlotJsonObject.percentileData)
		{
			var boxNum = i;
			var v1 = boxPlotJsonObject.percentileData[i].v1;
			var v5 = boxPlotJsonObject.percentileData[i].v5;
			if (v1 > maxValue)
			{
				maxValue = v1;
			}
			if (v5 > maxValue)
			{
				maxValue = v5;
			}
		}
		return maxValue;
	}

	/**
	 *	Draws centered top header title in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawHeaderTitleTopCenter(text, text_top_off, context, xoff, yoff, wdt, hgt)
	{
		// Add centered top header title
		context.font = '16px sans-serif';
		context.textBaseline = 'top';
		context.textAlign="center";
		context.fillText(text, xoff + wdt/2, text_top_off);
	}
	 
	/**
	 *	Draws centered top title in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawTitleTopCenter(text, text_axis_off, context, xoff, yoff, wdt, hgt)
	{
		// Add centered top title
		var text_axis_yoff = -text_axis_off;
		context.font = '15px sans-serif';
		context.textBaseline = 'bottom';
		context.textAlign="center";
		context.fillText(text, xoff + wdt/2, yoff + text_axis_yoff);
	}
	 
	/**
	 *	Draws centered bottom title in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawTitleBottomCenter(text, text_axis_off, context, xoff, yoff, wdt, hgt)
	{
		// Add centered bottom title
		var text_axis_yoff = text_axis_off;
		context.font = '15px sans-serif';
		context.textBaseline = 'top';
		context.textAlign="center";
		context.fillText(text, xoff + wdt/2, yoff + hgt + text_axis_yoff);
	}
	 
	/**
	 *	Draws right-aligned top sub-title in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawSubTitleTopRight(text, text_axis_off, context, xoff, yoff, wdt, hgt)
	{
		// Add right-aligned top title
		var text_axis_yoff = -text_axis_off;
		context.font = '12px sans-serif';
		context.textBaseline = 'bottom';
		context.textAlign="right";
		context.fillText(text, xoff + wdt, yoff + text_axis_yoff);
	}
	 
	/**
	 *	Draws left-aligned top sub-title in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawSubTitleTopLeft(text, sub_title_num, text_axis_off, context, xoff, yoff, wdt, hgt)
	{
		// Add left-aligned top title
		var text_axis_yoff = -text_axis_off;
		//var text_yoff = 35 - 11*(2 - sub_title_num);
		var text_axis_yoff = -text_axis_off;
		var text_yoff = yoff + text_axis_yoff - 11*(2 - sub_title_num);
		context.font = '10px sans-serif';
		context.textBaseline = 'bottom';
		context.textAlign="left";
		context.fillText(text, xoff, text_yoff);
	}
	 
	/**
	 *	Draws vertical box plot box with five percentiles.
	 *	Box numbers starts from 0 at the left.
	 */
	function drawBoxPlotBoxY(boxNum, boxNumTot, v1, v2, v3, v4, v5, min, max, min_off, max_off, x_pad, context, xoff, yoff, wdt, hgt)
	{
		// Calculate value offsets
		var eff_hgt = hgt - min_off - max_off;
		var value_range = max - min;
		var v1_yoff = min_off + eff_hgt*((v1 - min)/value_range);
		var v2_yoff = min_off + eff_hgt*((v2 - min)/value_range);
		var v3_yoff = min_off + eff_hgt*((v3 - min)/value_range);
		var v4_yoff = min_off + eff_hgt*((v4 - min)/value_range);
		var v5_yoff = min_off + eff_hgt*((v5 - min)/value_range);
		// Calculate box width
		var width_per_box = (wdt - 2*x_pad)/boxNumTot;
		// Reserve 20% as space between boxes
		var box_pad = width_per_box/10;
		var box_wdt = width_per_box*4/5;
		// Calculate box min, center, and max offsets
		var box_xmin = x_pad + boxNum*width_per_box + box_pad;
		var box_xcenter = box_xmin + box_wdt/2;
		var box_xmax = box_xmin + box_wdt;
		// Draw box rectangle
		drawRect(box_xmin, v2_yoff, box_wdt, v4_yoff - v2_yoff, context, xoff, yoff, wdt, hgt);
		// Draw median line (thick)
		context.lineWidth = 2;
		context.beginPath();
		drawLine(box_xmin, v3_yoff, box_xmax, v3_yoff, context, xoff, yoff, wdt, hgt);
		context.stroke();
		// Draw line between first percentile and box (dashed line)
		context.lineWidth = 1;
		context.beginPath();
		//drawLine(box_xcenter, v1_yoff, box_xcenter, v2_yoff, context, xoff, yoff, wdt, hgt);
		drawDashedLineVertical(box_xcenter, v1_yoff, v2_yoff, context, xoff, yoff, wdt, hgt);
		// Draw short horizontal line at end of line
		drawLine(box_xcenter - box_wdt/4, v1_yoff, box_xcenter + box_wdt/4, v1_yoff, context, xoff, yoff, wdt, hgt);		
		context.stroke();
		// Draw line between box and fifth percentile (dashed line)
		context.lineWidth = 1;
		context.beginPath();
		//drawLine(box_xcenter, v4_yoff, box_xcenter, v5_yoff, context, xoff, yoff, wdt, hgt);
		drawDashedLineVertical(box_xcenter, v4_yoff, v5_yoff, context, xoff, yoff, wdt, hgt);
		// Draw short horizontal line at end of line
		drawLine(box_xcenter - box_wdt/4, v5_yoff, box_xcenter + box_wdt/4, v5_yoff, context, xoff, yoff, wdt, hgt);		
		context.stroke();
	}
	 
	/**
	 *	Draws x-axis bottom scale marker with text in a canvas context
	 *	for a box plot box, using coordinates where x increases to the right
	 *	and y increases upwards, and where the offsets are added automatically.
	 */
	function drawBoxPlotBoxScaleMarkerXBottom(boxNum, boxNumTot, value, x_pad, context, xoff, yoff, wdt, hgt)
	{
		// Calculate box width
		var width_per_box = (wdt - 2*x_pad)/boxNumTot;
		// Reserve 20% as space between boxes
		var box_pad = width_per_box/10;
		var box_wdt = width_per_box*4/5;
		// Calculate box min, center, and max offsets
		var box_xmin = x_pad + boxNum*width_per_box + box_pad;
		var box_xcenter = box_xmin + box_wdt/2;
		var box_xmax = box_xmin + box_wdt;
		// Draw scale marker
		var marker_len = 10;
		context.lineWidth = 1;
		context.beginPath();
		drawLine(box_xcenter, 0, box_xcenter, -marker_len, context, xoff, yoff, wdt, hgt);
		context.stroke();
		// Draw scale marker value
		var value_text = '' + value;
		var text_axis_yoff = 20;
		var text_marker_xoff = 5;
		context.save();
		context.font = '10px sans-serif';
		context.textBaseline = 'bottom';
		context.translate(xoff + box_xcenter + text_marker_xoff, yoff + hgt + text_axis_yoff);
		context.rotate(-Math.PI/2);
		context.textAlign = "right";
		context.fillText(value_text, 0, 0);
		context.restore();
	}
	 
	/**
	 *	Draws x-axis top scale marker with text in a canvas context
	 *	for a box plot box, using coordinates where x increases to the right
	 *	and y increases upwards, and where the offsets are added automatically.
	 */
	function drawBoxPlotBoxScaleMarkerXTop(boxNum, boxNumTot, value, x_pad, context, xoff, yoff, wdt, hgt)
	{
		// Calculate box width
		var width_per_box = (wdt - 2*x_pad)/boxNumTot;
		// Reserve 20% as space between boxes
		var box_pad = width_per_box/10;
		var box_wdt = width_per_box*4/5;
		// Calculate box min, center, and max offsets
		var box_xmin = x_pad + boxNum*width_per_box + box_pad;
		var box_xcenter = box_xmin + box_wdt/2;
		var box_xmax = box_xmin + box_wdt;
		// Draw scale marker
		var marker_len = 10;
		context.lineWidth = 1;
		context.beginPath();
		drawLine(box_xcenter, hgt, box_xcenter, hgt + marker_len, context, xoff, yoff, wdt, hgt);
		context.stroke();
		// Draw scale marker value
		var value_text = '' + value;
		var text_axis_yoff = 20;
		var text_marker_xoff = 5;
		context.save();
		context.font = '10px sans-serif';
		context.textBaseline = 'bottom';
		context.translate(xoff + box_xcenter + text_marker_xoff, yoff - text_axis_yoff);
		context.rotate(-Math.PI/2);
		context.textAlign = "left";
		context.fillText(value_text, 0, 0);
		context.restore();
	}
	 
	/**
	 *	Draws y-axis left scale marker with text in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawScaleMarkerYLeft(value, markerText, min, max, min_off, max_off, context, xoff, yoff, wdt, hgt)
	{
		var eff_hgt = hgt - min_off - max_off;
		var value_range = max - min;
		var marker_yoff = min_off + eff_hgt*((value - min)/value_range);
		var marker_len = 10;
		// Draw scale marker
		context.lineWidth = 1;
		context.beginPath();
		drawLine(0, marker_yoff, -marker_len, marker_yoff, context, xoff, yoff, wdt, hgt);
		context.stroke();
		// Draw scale marker value
		var value_text = '' + markerText;
		var text_axis_xoff = -20;
		var text_marker_yoff = -5;
		context.font = '10px sans-serif';
		context.textBaseline = 'bottom';
		context.textAlign="right";
		drawText(value_text, text_axis_xoff, marker_yoff + text_marker_yoff, context, xoff, yoff, wdt, hgt);
	}
	 
	/**
	 *	Draws y-axis right scale marker with text in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawScaleMarkerYRight(value, markerText, min, max, min_off, max_off, context, xoff, yoff, wdt, hgt)
	{
		var eff_hgt = hgt - min_off - max_off;
		var value_range = max - min;
		var marker_yoff = min_off + eff_hgt*((value - min)/value_range);
		var marker_len = 10;
		// Draw scale marker
		context.lineWidth = 1;
		context.beginPath();
		drawLine(wdt, marker_yoff, wdt + marker_len, marker_yoff, context, xoff, yoff, wdt, hgt);
		context.stroke();
		// Draw scale marker value
		var value_text = '' + markerText;
		var text_axis_xoff = 20;
		var text_marker_yoff = -5;
		context.font = '10px sans-serif';
		context.textBaseline = 'bottom';
		context.textAlign="left";
		drawText(value_text, wdt + text_axis_xoff, marker_yoff + text_marker_yoff, context, xoff, yoff, wdt, hgt);
	}
	 
	/**
	 *	Draws dotted line for y-value in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawDottedLineY(value, min, max, min_off, max_off, context, xoff, yoff, wdt, hgt)
	{
		var eff_hgt = hgt - min_off - max_off;
		var value_range = max - min;
		var line_yoff = min_off + eff_hgt*((value - min)/value_range);
		// Draw dotted line for y-value
		context.lineWidth = 1;
		context.beginPath();
		drawDottedLineHorizontal(0, wdt, line_yoff, context, xoff, yoff, wdt, hgt);
		context.stroke();
	}
	 
	/**
	 *	Draws y-axis left scale title in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawTitleYLeft(title, context, xoff, yoff, wdt, hgt)
	{
		// Add y axis title
		var text_axis_xoff = -50;
		context.save();
		context.font = '15px sans-serif';
		context.textBaseline = 'bottom';
		context.translate(xoff + text_axis_xoff, yoff + hgt/2);
		context.rotate(-Math.PI/2);
		context.textAlign = "center";
		context.fillText(title, 0, 0);
		context.restore();
	}
	 
	/**
	 *	Draws y-axis right scale title in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawTitleYRight(title, context, xoff, yoff, wdt, hgt)
	{
		// Add y axis title
		var text_axis_xoff = 50;
		context.save();
		context.font = '15px sans-serif';
		context.textBaseline = 'top';
		context.translate(xoff + wdt + text_axis_xoff, yoff + hgt/2);
		context.rotate(-Math.PI/2);
		context.textAlign = "center";
		context.fillText(title, 0, 0);
		context.restore();
	}
	 
	/**
	 *	Draws a rectangle in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 *	Rectangle input is given as coordinates for lower left corner
	 *	and rectangle width and height. 
	 */
	function drawRect(x, y, rect_wdt, rect_hgt, context, xoff, yoff, wdt, hgt)
	{
		var x_with_offset = x + xoff;
		var y_with_offset = yoff + hgt - y;
		// Draw box rectangle
		context.strokeStyle = "#000000";
		context.strokeRect(x_with_offset, y_with_offset - rect_hgt, rect_wdt, rect_hgt);
	}
	 
	/**
	 *	Draws a line in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawLine(x1, y1, x2, y2, context, xoff, yoff, wdt, hgt)
	{
		var x1_with_offset = x1 + xoff;
		var x2_with_offset = x2 + xoff;
		var y1_with_offset = yoff + hgt - y1;
		var y2_with_offset = yoff + hgt - y2;
		context.moveTo(x1_with_offset, y1_with_offset);
		context.lineTo(x2_with_offset, y2_with_offset);
	}
	 
	/**
	 *	Draws a dashed vertical line in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawDashedLineVertical(x, y1, y2, context, xoff, yoff, wdt, hgt)
	{
		var dash_step1 = 3;
		var space_step1 = 3;
		var sign = 1;
		if (y1 > y2)
		{
			sign = -1;
		}
		var y = y1;
		var y_new;
		while (sign*(y2 - y) > 0)
		{
			y_new = y + sign*dash_step1;
			if (sign*y_new > sign*y2)
			{
				y_new = y2;
			}
			drawLine(x, y, x, y_new, context, xoff, yoff, wdt, hgt);
			y = y_new + sign*space_step1;
		}
	}
	 
	/**
	 *	Draws a dotted horizontal line in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawDottedLineHorizontal(x1, x2, y, context, xoff, yoff, wdt, hgt)
	{
		var dash_step1 = 1;
		var space_step1 = 3;
		var sign = 1;
		if (x1 > x2)
		{
			sign = -1;
		}
		var x = x1;
		var x_new;
		while (sign*(x2 - x) > 0)
		{
			x_new = x + sign*dash_step1;
			if (sign*x_new > sign*x2)
			{
				x_new = x2;
			}
			drawLine(x, y, x_new, y, context, xoff, yoff, wdt, hgt);
			x = x_new + sign*space_step1;
		}
	}
	 
	/**
	 *	Draws text in a canvas context, using coordinates
	 *	where x increases to the right and y increases upwards,
	 *	and where the offsets are added automatically.
	 */
	function drawText(text, x1, y1, context, xoff, yoff, wdt, hgt)
	{
		var x1_with_offset = x1 + xoff;
		var y1_with_offset = yoff + hgt - y1;
		context.fillText(text, x1_with_offset, y1_with_offset);
	}

	/**
	 *	Calculates an array of values for y-axis scale markers (ticks),
	 *	given the min and max values of the data, and the numbers
	 *	of markers (optional).
	 *
	 *	Based on PHP example on
	 *	http://stackoverflow.com/questions/326679/choosing-an-attractive-linear-scale-for-a-graphs-y-axis
	 *	but with modified behavior for step size < 1.
	 */
	function calculateScaleMarkerArray(minDataValue, maxDataValue, numTicks)
	{
		if (numTicks == null || numTicks <= 0)
		{
			numTicks = 10;
		}
		// Adjust ticks if needed
		if (numTicks < 2)
		{
			numTicks = 2;
		}
		else if (numTicks > 2)
		{
			numTicks -= 2;
		}

		// If min and max are identical, then adjust these values.
		if (minDataValue == maxDataValue)
		{
			minDataValue = minDataValue - 10;
			maxDataValue = maxDataValue + 10;
		}

		// Determine range
		var range = maxDataValue - minDataValue;
		// Get raw step value
		var tempStep = range/numTicks;
		// Calculate pretty step value
		var mag = Math.floor(Math.log(tempStep)/Math.log(10) + 0.001);
		var magPow = Math.pow(10, mag);
		if (magPow >= 1)
		{
			magPow = Math.floor(magPow + 0.001);
		}
		var magMsd = Math.floor(tempStep/magPow + 0.5);
		var stepSize = magMsd*magPow;
		// Correction for stepSize < 1000
		if (stepSize < 1000)
		{
			if (stepSize >= 750)
			{
				stepSize = 1000;
			}
			else if (stepSize >= 250)
			{
				stepSize = 500;
			}
			else if (stepSize >= 75)
			{
				stepSize = 100;
			}
			else if (stepSize >= 25)
			{
				stepSize = 50;
			}
			else if (stepSize >= 7.5)
			{
				stepSize = 10;
			}
			else if (stepSize >= 2.5)
			{
				stepSize = 5;
			}
			else if (stepSize >= 0.75)
			{
				stepSize = 1;
			}
			else if (stepSize >= 0.25)
			{
				stepSize = 0.5;
			}
			else if (stepSize >= 0.075)
			{
				stepSize = 0.1;
			}
			else if (stepSize >= 0.025)
			{
				stepSize = 0.05;
			}
			else if (stepSize >= 0.0075)
			{
				stepSize = 0.01;
			}
			else if (stepSize >= 0.0025)
			{
				stepSize = 0.005;
			}
			else if (stepSize >= 0.00075)
			{
				stepSize = 0.001;
			}
			else if (stepSize >= 0.00025)
			{
				stepSize = 0.0005;
			}
			else if (stepSize >= 0.000075)
			{
				stepSize = 0.0001;
			}
		}
		
		// Build y label array
		// Lower and upper bounds calculations
		var lowerBound = stepSize*Math.floor(minDataValue/stepSize);
		var upperBound = stepSize*Math.ceil(maxDataValue/stepSize);
		// Build array
		var tickValuesArr = [];
		var value = lowerBound;
		var i = 0;
		tickValuesArr[i] = value;
		value += stepSize;
		while(value <= upperBound)
		{
			i++;
			tickValuesArr[i] = value;
			value += stepSize;
		}
		return tickValuesArr;
	}
	 
	/**
	 *	Trims an array of values for y-axis scale markers (ticks),
	 *	to only include values in given value range.
	 */
	function trimScaleMarkerArray(origTickValuesArr, minValue, maxValue)
	{
		// Build trimmed y label array
		var tickValuesArr = [];
		var value;
		var k = 0;
		for (i=0; i < origTickValuesArr.length; i++)
		{
			value = origTickValuesArr[i];
			if (value >= minValue && value <= maxValue)
			{
				// Add value to trimmed array
				tickValuesArr[k] = value;
				k++;
			}
		}
		return tickValuesArr;
	}
